# FILE: mmlaunch.tcl
#
# An application which presents an interface for launching and 
# interconnecting other OOMMF applications.
#
# Last modified on: $Date: 1999/06/30 21:17:51 $
# Last modified by: $Author: dgp $

# Support libraries
package require Ow
package require Net
package require Oc
if {[catch {package require Tk 8}]} {
    package require Tk 4.1
}

Oc_Main SetAppName mmLaunch
Oc_Main SetVersion 1.0.1.2
regexp \\\044Date:(.*)\\\044 {$Date: 1999/06/30 21:17:51 $} _ date
Oc_Main SetDate [string trim $date]
# regexp \\\044Author:(.*)\\\044 {$Author: dgp $} _ author
# Oc_Main SetAuthor [Oc_Person Lookup [string trim $author]]
Oc_Main SetAuthor [Oc_Person Lookup dgp]
Oc_Main SetHelpURL [Oc_Url FromFilename [file join [file dirname \
        [file dirname [file dirname [Oc_DirectPathname [info \
        script]]]]] doc userguide OOMMF_Launcher_Control_Inte.html]]

set menubar .mb
foreach {fmenu hmenu} [Ow_MakeMenubar . $menubar File Help] {}
$fmenu add command -label "Exit" -underline 1 -command exit
Ow_StdHelpMenu $hmenu

# Host checkbutton frame
set hframe [frame .omfHosts -bd 4 -relief ridge]
set hpanel [frame .omfPanel -relief groove]

set menuwidth [Ow_GuessMenuWidth $menubar]
set brace [canvas .brace -width $menuwidth -height 0 -borderwidth 0 \
        -highlightthickness 0]
pack $brace -side top

pack $hframe -side left -fill both -expand 1
pack $hpanel -side left -fill both -expand 1

wm title . [Oc_Main GetAppName]<[pid]>
wm iconname . [wm title .]
Ow_SetIcon .

if {[package vcompare [package provide Tk] 8] >=0 \
	&& [string match windows $tcl_platform(platform)]} {
    # Windows doesn't size Tcl 8.0 menubar cleanly
    Oc_DisableAutoSize .
    wm geometry . "${menuwidth}x0"
    update
    wm geometry . {}
    Oc_EnableAutoSize .
    update
}

proc DisplayHostButtons {} {
    global hframe hpanel
    # Destroy existing buttons
    foreach w [winfo children $hframe] {
        destroy $w
    }
    set displayedhost 0
    foreach host [Net_Host Instances] {
        if {[$host Ready]} {
            set btn [checkbutton $hframe.w$host -text [$host Cget -hostname] \
                -anchor w \
                -variable [$host GlobalName btnvar] \
                -command [list $host ToggleGui $hpanel]]
            pack $btn -side top -fill x
            set displayedhost 1
        }
    }
    wm geometry . {}
}
em_eventHandler new Net_Host Ready DisplayHostButtons
em_eventHandler new Net_Host delete DisplayHostButtons

# Listen for tell messages from all accounts -- they announce new threads
# Each time a new account becomes Ready create a listening Handler for it.
em_eventHandler new Net_Account Ready StartAccountListener
proc StartAccountListener {} {
    global listeners
    foreach acct [Net_Account Instances] {
        if {![info exists listeners($acct,r)]} {
            set listeners($acct,r) [em_eventHandler new $acct Readable \
                    [list HandleAccountTell $acct]]
            set listeners($acct,d) [em_eventHandler new $acct delete \
                    [list DeleteAccountListener $acct]]
        }
    }
}
proc DeleteAccountListener { acct } {
    global listeners
    if {[info exists listeners($acct,r)]} {
        $listeners($acct,r) delete
        $listeners($acct,d) delete
        unset listeners($acct,r)
        unset listeners($acct,d)
    }
}
proc HandleAccountTell { acct } {
    set message [$acct Get]
    switch -exact -- [lindex $message 0] {
        newthread {
            NewThread $acct
        }
        default {
          [em_extension ApplicationLog] Append "Bad message from account $acct:
	$message"
        }
    }
}
proc NewThread { acct } {
    global qhandler thandler
    if {[info exists qhandler($acct)]} {
        # Only one 'threads' request per account can use qhandler at a time
        after 1000 NewThread $acct
        return
    }
    set qid [$acct Send threads]
    set qhandler($acct) [em_eventHandler new $acct Reply$qid \
            [list NewThreadReply $acct]]
    set thandler($acct) [em_eventHandler new $acct Timeout$qid \
            [list NewThreadTimeout $acct]]
}
proc NewThreadReply { acct } {
    global qhandler thandler
    $qhandler($acct) delete
    $thandler($acct) delete
    unset qhandler($acct)
    unset thandler($acct)
    set threads [$acct Get]
    [$acct Cget -log] Append "Received thread list: $threads"
    # create a thread for any threads on list not already represented
    # This is a really dumb way to do this!
    if {![lindex $threads 0]} {
        foreach triple [lrange $threads 1 end] {
            Net_Thread New _ -hostname [$acct Cget -hostname] \
                    -accountname [$acct Cget -accountname] \
                    -pid [lindex $triple 1] -alias [lindex $triple 0] \
                    -log [em_extension ApplicationLog]
        }
    }
}
proc NewThreadTimeout { acct } {
    global qhandler thandler
    $qhandler($acct) delete
    $thandler($acct) delete
    unset qhandler($acct)
    unset thandler($acct)
    set msg "Timed out waiting for thread list"
    [$acct Cget -log] Append $msg
    error $msg $msg
}

# Create initial host list -- try to connect to account servers
set hostList localhost
Oc_Option Get LaunchInterface hostList hostList
foreach host $hostList {
    if {![string match $host* [info hostname]]} {
        Net_Host New _ -hostname $host
        Net_Account New _ -hostname $host \
                -log [em_extension ApplicationLog]
    }
}
if {[string match localhost [info hostname]]} {
    Net_Host New _ -hostname localhost
    Net_Account New _ -hostname localhost \
            -log [em_extension ApplicationLog]
}

# When any account becomes ready, get its threads
foreach ia [Net_Account Instances] {
    em_eventHandler new $ia Ready [list NewThread $ia]
}

